﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Algorithms.DataAnalysis;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class GaussianProcessRegressionInstance2D : ArtificialRegressionDataDescriptor {

    private const int STEPS = 20;
    public override string Name {
      get { return "Gaussian Process " + name; }
    }
    public override string Description {
      get { return ""; }
    }
    protected override string TargetVariable { get { return "Y"; } }
    protected override string[] VariableNames { get { return new string[] { "X1", "X2", "Y" }; } }
    protected override string[] AllowedInputVariables { get { return new string[] { "X1", "X2" }; } }
    protected override int TrainingPartitionStart { get { return 0; } }
    protected override int TrainingPartitionEnd { get { return (STEPS+1) * (STEPS+1); } }
    protected override int TestPartitionStart { get { return TrainingPartitionEnd; } }
    protected override int TestPartitionEnd { get { return 2 * (STEPS + 1) * (STEPS + 1); } }

    private ParameterizedCovarianceFunction cov;
    private string name;

    public GaussianProcessRegressionInstance2D(string name, ICovarianceFunction covarianceFunction, double[] hyp) {
      this.name = name;
      cov = covarianceFunction.GetParameterizedCovarianceFunction(hyp, Enumerable.Range(0, 2));
    }

    protected override List<List<double>> GenerateValues() {
      List<List<double>> trainingData = new List<List<double>>() {
        ValueGenerator.GenerateSteps(0, 1, 1.0 / STEPS).ToList(),
        ValueGenerator.GenerateSteps(0, 1, 1.0 / STEPS).ToList()
      };

      List<List<double>> testData = new List<List<double>>() {
        ValueGenerator.GenerateSteps(-0.1, 1.1, 1.2 / STEPS).ToList(),
        ValueGenerator.GenerateSteps(-0.1, 1.1, 1.2 / STEPS).ToList()
      };

      var trainingComb = ValueGenerator.GenerateAllCombinationsOfValuesInLists(trainingData).ToList<IEnumerable<double>>();
      var testComb = ValueGenerator.GenerateAllCombinationsOfValuesInLists(testData).ToList<IEnumerable<double>>();

      List<List<double>> data = new List<List<double>>();
      for (int i = 0; i < AllowedInputVariables.Count(); i++) {
        data.Add(trainingComb[i].ToList());
        data[i].AddRange(testComb[i]);
      }

      var mt = new MersenneTwister();

      var target = Util.SampleGaussianProcess(mt, cov, data);
      data.Add(target);

      return data;
    }
  }
}
